﻿using Infragistics.Controls.Schedules;
using Infragistics.Controls.Schedules.Primitives;
using Infragistics.Windows;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml.Linq;

namespace GanttRowStyleForSelection
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            Loaded += MainWindow_Loaded;
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            //ProjectTableGrid grid = (ProjectTableGrid)Utilities.GetDescendantFromType(gantt, typeof(ProjectTableGrid), false);
            //if (grid != null)
            //{
            //    var existingStyle = grid.CellStyle;
            //    var currentStyle = LayoutRoot.Resources["CellMarkingStyle"] as Style;
            //    currentStyle.BasedOn = existingStyle;
            //    grid.CellStyle = currentStyle;
            //}
        }
    }

    public class ProjectDataHelper
    {
        public static Project GenerateProjectData()
        {
            // Create a Project instance
            Project project = new Project();
            // Create a project root ProjectTask
            ProjectTask rootTask = new ProjectTask
            {
                TaskName = "Project Summary Task",
                IsManual = false
            };
            // Add the root task to the project
            project.RootTask.Tasks.Add(rootTask);
            // Dates in xamGantt are stored in UTC
            DateTime startTime = DateTime.Today.ToUniversalTime();
            // Add ProjectTask subtasks to the root project task
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Planning",
                IsManual = false,
                Start = startTime,
                Duration = TimeSpan.FromHours(8),
            });
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Documenting",
                IsManual = false,
                Start = startTime,
                // Configure task duration
                ManualDuration = ProjectDuration.FromFormatUnits(2, ProjectDurationFormat.Days),
                // Set a task constraint
                ConstraintDate = DateTime.Today.AddDays(3).ToUniversalTime(),
                ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan,
            });
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Creating Demo Application",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(3, ProjectDurationFormat.Days),
                ConstraintType = ProjectTaskConstraintType.MustStartOn,
                ConstraintDate = DateTime.Today.AddDays(2).ToUniversalTime(),
                IsEstimated = true,
            });
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Collecting Feedback",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(1, ProjectDurationFormat.Days),
                IsEstimated = true,
            });
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Designing",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(5, ProjectDurationFormat.Days),
            });
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Development",
                IsManual = true,
                // Manual tasks may have free text for Start and Finish date
                ManualStart = new ManualDateTime("Still In Discussion"),
                ManualDuration = ProjectDuration.FromFormatUnits(10, ProjectDurationFormat.Days),
                ManualFinish = new ManualDateTime("Not Defined Yet"),
            });
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Testing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(7, ProjectDurationFormat.Days),
            });
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Bug Fixing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(4, ProjectDurationFormat.Days),
            });
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Project Complete",
                IsManual = false,
                Start = startTime,
                // Set a milestone
                IsMilestone = true
            });
            return project;
        }
    }

    public class TaskModel : ObservableModel
    {
        public string TaskID { get; set; }
        public string Tasks { get; set; }
        public string TaskName { get; set; }
        public bool IsManual { get; set; }
        public DateTime Start { get; set; }
        public TimeSpan Duration { get; set; }
        public ProjectDuration ManualDuration { get; set; }
        public DateTime ConstraintDate { get; set; }
        public ProjectTaskConstraintType ConstraintType { get; set; }
        public bool IsEstimated { get; set; }
        public ManualDateTime ManualStart { get; set; }
        public ManualDateTime ManualFinish { get; set; }
        public bool IsMilestone { get; set; }
        public ProjectDurationFormat DurationFormat { get; set; }

        public bool IsMarked { get; set; }
    }

    public class ListBackedProjectViewModel : ObservableModel
    {
        public ListBackedProjectViewModel()
        {
            this.DownloadDataSource();
        }
        private ObservableCollection<TaskModel> _tasks;
        public ObservableCollection<TaskModel> Tasks
        {
            get
            {
                return _tasks;
            }
            set
            {
                if (value != null)
                {
                    _tasks = value;
                }
                NotifyPropertyChanged("Tasks");
            }
        }
        private void DownloadDataSource()
        {
            ObservableCollection<TaskModel> dataSource = new ObservableCollection<TaskModel>();

            DateTime startTime = DateTime.Today.ToUniversalTime();

            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Planning",
                IsManual = false,
                Start = startTime,
                Duration = TimeSpan.FromHours(8),
            });
            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Documenting",
                IsManual = false,
                Start = startTime,
                // Configure task duration
                ManualDuration = ProjectDuration.FromFormatUnits(2, ProjectDurationFormat.Days),
                // Set a task constraint
                ConstraintDate = DateTime.Today.AddDays(3).ToUniversalTime(),
                ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan,
                IsMarked = true,
            });
            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Creating Demo Application",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(3, ProjectDurationFormat.Days),
                ConstraintType = ProjectTaskConstraintType.MustStartOn,
                ConstraintDate = DateTime.Today.AddDays(2).ToUniversalTime(),
                IsEstimated = true,
            });
            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Collecting Feedback",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(1, ProjectDurationFormat.Days),
                IsEstimated = true,
                IsMarked = true,
            });
            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Designing",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(5, ProjectDurationFormat.Days),
            });
            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Development",
                IsManual = true,
                // Manual tasks may have free text for Start and Finish date
                ManualStart = new ManualDateTime("Still In Discussion"),
                ManualDuration = ProjectDuration.FromFormatUnits(10, ProjectDurationFormat.Days),
                ManualFinish = new ManualDateTime("Not Defined Yet"),
            });
            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Testing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(7, ProjectDurationFormat.Days),
            });
            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Bug Fixing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(4, ProjectDurationFormat.Days),
            });
            dataSource.Add(new TaskModel
            {
                TaskID = Guid.NewGuid().ToString(),
                TaskName = "Project Complete",
                IsManual = false,
                Start = startTime,
                // Set a milestone
                IsMilestone = true
            });

            this._tasks = dataSource;
        }
    }
    public class ObservableModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
    }
}
